from shipday.order.address import Address
from shipday.order.customer import Customer
from shipday.order.order_cost import OrderCost
from shipday.order.order_info import Order
from shipday.order.order_item import OrderItem
from shipday.order.order_query import OrderQuery
from shipday.order.order_status import OrderStatus
from shipday.order.pickup import Pickup



from collections import defaultdict

from shipday.utils.verifiers import verify_not_negative


class OrderCost:
    def __init__(self, *args,
                 tips: float = 0.0, tax: float = 0.0, discount: float = 0.0, delivery_fee: float = 0.0,
                 total: float = 0.0,
                 **kwargs):
        kwargs = defaultdict(lambda: 0.0, **kwargs)
        self._tips = tips
        self._tax = tax
        self._discount = discount or kwargs['discountAmount']
        self._delivery_fee = delivery_fee or kwargs['deliveryFee']
        self._total = total or kwargs['totalOrderCost']

    @property
    def tips(self):
        return self._tips

    @tips.setter
    def tips(self, value):
        verify_not_negative(value, "Tips must be a positive number")
        self._tips = float(value)

    @property
    def tax(self):
        return self._tax

    @tax.setter
    def tax(self, value):
        verify_not_negative(value, "Tax must be a positive number")
        self._tax = value

    @property
    def discount(self):
        return self._discount

    @discount.setter
    def discount(self, value):
        verify_not_negative(value, "Discount must be a positive number")
        self._discount = value

    @property
    def delivery_fee(self):
        return self._delivery_fee

    @delivery_fee.setter
    def delivery_fee(self, value):
        verify_not_negative(value, "Delivery Fee must be a positive number")
        self._delivery_fee = float(value)

    @property
    def total(self):
        return self._total

    @total.setter
    def total(self, value):
        verify_not_negative(value, "Total must be a positive number")
        self._total = value

    def verify(self):
        verify_not_negative(self._tips, "Tips must be a positive number")
        verify_not_negative(self._tax, "Tax must be a positive number")
        verify_not_negative(self._discount, "Discount must be a positive number")
        verify_not_negative(self._delivery_fee, "Delivery Fee must be a positive number")
        verify_not_negative(self._total, "Total must be a positive number")

    def get_body(self):
        return {
            'tips': self.tips,
            'tax': self.tax,
            'discountAmount': self.discount,
            'deliveryFee': self.delivery_fee,
            'totalCost': self.total,
            'totalOrderCost': self.total,
        }



import json
from collections import defaultdict

from shipday.order.address import Address
from shipday.utils.verifiers import verify_instance_of, verify_none_or_instance_of


class Customer:
    def __init__(self, *arg,
                 name: str = None, address: Address = None, email: str = None, phone_number: str = None,
                 **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        self._name = name or kwargs['customerName']
        self._address = address
        self._email = email or kwargs['customerEmail']
        self._phone_number = phone_number or kwargs['customerPhoneNumber']
        self._address_line = kwargs['customerAddress']

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        verify_instance_of(str, value, "Name is not String")
        self._name = value

    @property
    def address(self) -> Address:
        return self._address

    @address.setter
    def address(self, value: Address):
        verify_instance_of(Address, value, "Address is not String")
        self._address = value

    @property
    def email(self):
        return self._email

    @email.setter
    def email(self, value):
        verify_instance_of(str, value, "Email is not String")
        self._email = value

    @property
    def phone_number(self):
        return self._phone_number

    @phone_number.setter
    def phone_number(self, value):
        verify_instance_of(str, value, "Phone number is not String")
        self._phone_number = value

    def __repr__(self):
        return json.dumps(self.get_body())

    def verify(self) -> None:
        verify_instance_of(str, self.name, "Customer must have a name")
        verify_instance_of(Address, self.address, "Customer must have a Address")
        verify_instance_of(str, self.phone_number, "Customer must have a phone number")
        verify_none_or_instance_of(str, self.email, "Customer email must be a String or None")

    def get_body(self) -> dict:
        obj = {
            'customerName': self.name,
            'customerAddress': self._address_line or (
                self.address.get_single_line() if self._address is not None else None),
            'dropoff': self.address.get_breakdown() if self._address is not None else None,
            'customerEmail': self.email,
            'customerPhoneNumber': self.phone_number
        }

        if self.address is not None and self.address.latitude is not None and self.address.longitude is not None:
            obj['deliveryLatitude'] = self.address.latitude
            obj['deliveryLongitude'] = self.address.longitude

        return obj



import json
from collections import defaultdict

from shipday.order.address import Address
from shipday.utils.verifiers import verify_instance_of, verify_none_or_instance_of


class Pickup:
    def __init__(self, *args,
                 name: str = None, address: Address = None, phone_number: str = None,
                 **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        self._name = name or kwargs['restaurantName']
        self._address = address
        self._phone_number = phone_number or kwargs['restaurantPhoneNumber']
        self.__address_line = kwargs['restaurantAddress']

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        verify_instance_of(str, value, 'Pickup name is not a string')
        self._name = value

    @property
    def address(self):
        return self._address

    @address.setter
    def address(self, value):
        verify_instance_of(Address, value, 'Pickup address is not of Address type')
        self._address = value

    @property
    def phone_number(self):
        return self._phone_number

    @phone_number.setter
    def phone_number(self, value):
        verify_instance_of(str, value, 'Pickup Phone number is not a string')
        self._phone_number = value

    def __repr__(self):
        return json.dumps(self.get_body())

    def verify(self):
        verify_instance_of(str, self._name, 'Pickup must have a name')
        verify_instance_of(Address, self._address, 'Pickup must be of type Address')
        verify_none_or_instance_of(str, self._phone_number, 'Pickup phone number must be String or None')

    def get_body(self) -> dict:
        obj = {
            'restaurantName': self.name,
            'restaurantAddress': self.__address_line or (
                self.address.get_single_line() if self._address is not None else None),
            'pickup': self.address.get_breakdown() if self._address is not None else None,
        }

        if self.address is not None and self.address.latitude is not None and self.address.longitude is not None:
            obj['pickupLatitude'] = self.address.latitude
            obj['pickupLongitude'] = self.address.longitude

        if self.phone_number is not None:
            obj['restaurantPhoneNumber'] = self.phone_number
        return obj



import json
from collections import defaultdict

from shipday.exeptions.shipday_exeption import ShipdayException
from shipday.utils.verifiers import verify_instance_of, verify_none_or_instance_of


class OrderItem:
    def __init__(self, name=None, unit_price=None, quantity=None, add_ons=None, detail=None,
                 **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        self._name = name
        self._unit_price = unit_price or kwargs['unitPrice'] or 0
        self._quantity = quantity
        self._add_ons = add_ons or kwargs['addOns']
        self._detail = detail

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value: str):
        verify_instance_of(str, value, 'OrderItem is not a name')
        self._name = value

    @property
    def unit_price(self):
        return self._unit_price

    @unit_price.setter
    def unit_price(self, value):
        if type(value) not in (float, int) or value < 0:
            raise ShipdayException("OrderItem must have a valid price")
        self._unit_price = value

    @property
    def quantity(self):
        return self._quantity

    @quantity.setter
    def quantity(self, value: int):
        if type(value) is not int or value < 1:
            raise ShipdayException("Order quantity must be a positive integer")
        self._quantity = value

    @property
    def add_ons(self):
        return self._add_ons

    @add_ons.setter
    def add_ons(self, value):
        verify_instance_of(str, value, 'add ons is not string')
        self._add_ons = value

    @property
    def detail(self):
        return self._detail

    @detail.setter
    def detail(self, value):
        verify_instance_of(str, value, 'detail is not string')
        self._detail = value

    def __repr__(self):
        return json.dumps(self.get_body())

    def verify(self):
        verify_instance_of(str, self._name, 'OrderItem must have a name of type string')
        verify_instance_of([int, float], self._unit_price, 'OrderItem must have a valid price of type int or float')
        verify_instance_of(int, self._quantity, 'Order quantity must be a positive integer')
        verify_none_or_instance_of(str, self._add_ons, 'Add ons must be String or None')
        verify_none_or_instance_of(str, self._detail, 'Details must be String or None')

    def get_body(self) -> dict:
        obj = {
            'name': self.name,
            'unitPrice': self.unit_price,
            'quantity': self.quantity
        }

        if self.add_ons is not None:
            obj['addOns'] = self.add_ons

        if self.detail is not None:
            obj['detail'] = self.detail

        return obj



from collections import defaultdict

from shipday.exeptions.shipday_exeption import ShipdayException
from shipday.utils.verifiers import verify_instance_of


class CarrierRequest:
    def __init__(self,
                 name:str = None, email:str = None, phone_number:str = None
                 , **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        self._name = name
        self._email = email
        self._phone_number = phone_number

    @property
    def name(self) -> str:
        return self._name

    @name.setter
    def name(self, value: str) -> None:
        verify_instance_of(str, value, "Carrier name is not a string")
        self._name = value

    @property
    def email(self) -> str:
        return self._email

    @email.setter
    def email(self, value: str) -> None:
        verify_instance_of(str, value, "Carrier email is not a string")
        self._email = value

    @property
    def phone_number(self) -> str:
        return self._phone_number

    @phone_number.setter
    def phone_number(self, value: str) -> None:
        verify_instance_of(str, value, "Carrier phone number is not a string")
        self._phone_number = value

    def verify(self) -> None:
        if self.name is None:
            raise ShipdayException('Carrier must have a name')
        if self.email is None:
            raise ShipdayException('Carrier must have an email')
        if self.phone_number is None:
            raise ShipdayException('Carrier must have a phone number')

    def get_body(self) -> dict:
        self.verify()
        return {
            'name': self.name,
            'phoneNumber': self.phone_number,
            'email': self.email
        }



from shipday.exeptions import ShipdayException


def verify_none_or_instance_of(obj_type, variable, error_message):
    if type(obj_type) is not list:
        obj_type = [obj_type]
    if variable is None:
        return
    if type(variable) not in obj_type:
        raise ShipdayException(error_message)


def verify_instance_of(obj_type, variable, error_message):
    if type(obj_type) is not list:
        obj_type = [obj_type]
    if type(variable) not in obj_type:
        raise ShipdayException(error_message)


def verify_not_negative(number, error_message):
    verify_instance_of([int, float], number, error_message)
    if number < 0:
        raise ShipdayException(error_message)


def verify_none_or_not_negative(number, error_message):
    verify_none_or_instance_of([int, float], number, error_message)

    if number is not None and number < 0:
        raise ShipdayException(error_message)


def verify_none_or_within_range(number, min_value, max_value, error_message):
    if number is None:
        return
    verify_instance_of([int, float], number, error_message)
    if number < min_value or number > max_value:
        raise ShipdayException(error_message)


def verify_all_none_or_not(variables, error_message):
    if variables is None:
        return
    if type(variables) is not list:
        variables = [variables]
    filtered = list(filter(None, variables))
    if len(filtered) != 0 and len(filtered) != len(variables):
        raise ShipdayException(error_message)



from shipday.exeptions import ShipdayException
from shipday.services import OrderService, CarrierService, OnDemandDeliveryService


class Shipday:
    def __init__(self, *args, api_key, **kwargs):
        self.__api_key__ = api_key
        self.__verify_api_key()
        self.OrderService = OrderService(api_key=api_key)
        self.CarrierService = CarrierService(api_key=api_key)
        self.OnDemandDeliveryService = OnDemandDeliveryService(api_key=api_key)

    def __verify_api_key(self):
        if self.__api_key__ is None or len(self.__api_key__) < 10:
            raise ShipdayException('Invalid API key')



from collections import defaultdict
from datetime import datetime
from typing import List

from shipday.exeptions import ShipdayException
from shipday.order.customer import Customer
from shipday.order.pickup import Pickup
from shipday.order.order_item import OrderItem
from shipday.order.order_cost import OrderCost
from shipday.utils.verifiers import verify_instance_of, verify_none_or_instance_of


class Order:
    def __init__(self, *args, order_number: str = None, customer: Customer = None, pickup: Pickup = None,
                 order_items: List[OrderItem] = None, order_cost: OrderCost = None,
                 expected_delivery_time: datetime = None,
                 expected_pickup_time: datetime = None,
                 **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        if type(customer) is not Customer:
            customer = None
        if type(pickup) is not Pickup:
            pickup = None
        if type(order_cost) is not OrderCost:
            order_cost = None
        if type(expected_delivery_time) is not datetime:
            expected_delivery_time = None
        if type(expected_pickup_time) is not datetime:
            expected_pickup_time = None
        if type(order_items) is not list:
            order_items = None
        else:
            for item in order_items:
                if type(item) is not OrderItem:
                    order_items = None
                    break
        self._order_number: str = order_number or kwargs['orderNumber']
        self._customer: Customer = customer or Customer(**kwargs)
        self._pickup: Pickup = pickup or Pickup(**kwargs)
        self._order_items: List[OrderItem] = order_items or list(
            map(lambda item: OrderItem(**item), kwargs['orderItems'] or []))
        self._order_cost: OrderCost = order_cost or OrderCost(**kwargs)
        self._delivery_time: datetime = expected_delivery_time
        self._pickup_time: datetime = expected_pickup_time
        self._delivery_instruction = kwargs['deliveryInstruction'] or kwargs['delivery_instruction']
        self._pickup_instruction = kwargs['pickupInstruction'] or kwargs['pickup_instruction']

    @property
    def order_number(self) -> str:
        return self._order_number

    @order_number.setter
    def order_number(self, value: str):
        verify_instance_of(str, value, 'Order number must be string')
        self._order_number = value

    @property
    def customer(self) -> Customer:
        return self._customer

    @customer.setter
    def customer(self, value: Customer):
        verify_instance_of(Customer, value, 'customer must be of type ' + str(Customer))
        self._customer = value

    @property
    def pickup(self) -> Pickup:
        return self._pickup

    @pickup.setter
    def pickup(self, value):
        verify_instance_of(Pickup, value, 'Pickup must be of type ' + str(Pickup))
        self._pickup = value

    @property
    def order_items(self) -> List[OrderItem]:
        return self._order_items

    @order_items.setter
    def order_items(self, value: List[OrderItem]):
        verify_none_or_instance_of(list, value, 'Orderitems must be a list or None')
        for item in value:
            verify_instance_of(OrderItem, item, 'Orderitems must be a list of OrderItem')
        self._order_items = value

    @property
    def order_cost(self) -> OrderCost:
        return self._order_cost

    @order_cost.setter
    def order_cost(self, value: OrderCost):
        verify_instance_of(OrderCost, value, 'Order must have OrderCost object')
        self._order_cost = value

    @property
    def expected_delivery_time(self):
        return self._delivery_time

    @expected_delivery_time.setter
    def expected_delivery_time(self, value):
        verify_none_or_instance_of(datetime, value, 'Delivery time is not of type ' + str(datetime))
        self._delivery_time = value

    @property
    def expected_pickup_time(self):
        return self._pickup_time

    @expected_pickup_time.setter
    def expected_pickup_time(self, value):
        verify_none_or_instance_of(datetime, value, 'Pickup time is not of type ' + str(datetime))
        self._pickup_time = value

    @property
    def delivery_instruction(self):
        return self._delivery_instruction

    @delivery_instruction.setter
    def delivery_instruction(self, value: str):
        verify_none_or_instance_of(str, value, 'Delivery Instruction must be of type string')
        self._delivery_instruction = value

    @property
    def pickup_instruction(self):
        return self._pickup_instruction

    @pickup_instruction.setter
    def pickup_instruction(self, value: str):
        verify_none_or_instance_of(str, value, 'Pickup Instruction must be of type string')
        self._pickup_instruction = value

    def __repr__(self):
        self.get_body()

    def update_total_cost(self):
        total = self._order_cost.tax + self._order_cost.tips + self._order_cost.delivery_fee - self._order_cost.discount
        try:
            for item in self.order_items:
                total += item.unit_price * item.quantity
        except ShipdayException:
            pass
        self._order_cost.total = total

    def verify(self):
        verify_instance_of(str, self._order_number, 'Order must have a order number')
        self._customer.verify()
        self._pickup.verify()
        for i, item in enumerate(self._order_items):
            try:
                verify_instance_of(OrderItem, item, 'Order Item is not of type OrderItem')
                item.verify()
            except ShipdayException as e:
                raise ShipdayException('Exception in item no: {} + "\n" + Message: {}'.format(i, str(e)))

    def get_body(self):
        obj = {
            'orderNumber': self.order_number,
            'orderItem': list(map(lambda x: x.get_body(), self.order_items)),
            **self.order_cost.get_body()
        }

        if self._customer is not None:
            obj.update(self.customer.get_body())

        if self._pickup is not None:
            obj.update(self.pickup.get_body())

        if self._order_cost is not None:
            obj.update(self.order_cost.get_body())

        if self.expected_delivery_time is not None:
            obj['expectedDeliveryDate'] = self.expected_delivery_time.date().isoformat()
            obj['expectedDeliveryTime'] = self.expected_delivery_time.time().isoformat(timespec='seconds')

        if self.expected_pickup_time is not None:
            obj['expectedPickupTime'] = self.expected_pickup_time.time().isoformat(timespec='seconds')

        if self._delivery_instruction is not None:
            obj['deliveryInstruction'] = self._delivery_instruction

        if self._pickup_instruction is not None:
            obj['pickupInstruction'] = self._pickup_instruction

        return obj



from collections import defaultdict

from shipday.utils.verifiers import verify_none_or_instance_of, verify_none_or_within_range, \
    verify_all_none_or_not, verify_instance_of


class Address:
    def __init__(self, *args, unit: str = None, street: str = None, city: str = None,
                 state: str = None, zip: str = None, country: str = None,
                 latitude: float = None, longitude: float = None,
                 **kwargs):
        kwargs = defaultdict(lambda: None, **kwargs)
        self._unit = unit
        self._street = street
        self._city = city
        self._state = state
        self._zip = zip
        self._country = country
        self._latitude = latitude
        self._longitude = longitude
        self._unit_in_address = kwargs['unit_in_address'] if 'unit_in_address' in kwargs else False

    @property
    def unit(self):
        return self._unit

    @unit.setter
    def unit(self, value):
        self._unit = value

    @property
    def street(self):
        return self._street

    @street.setter
    def street(self, value):
        self._street = value

    @property
    def city(self):
        return self._city

    @city.setter
    def city(self, value: str):
        self._city = value

    @property
    def state(self):
        return self._state

    @state.setter
    def state(self, value):
        self._state = value

    @property
    def zip(self):
        return self._zip

    @zip.setter
    def zip(self, value):
        self._zip = value

    @property
    def country(self):
        return self._country

    @country.setter
    def country(self, value):
        self._country = value

    @property
    def latitude(self):
        return self._latitude

    @latitude.setter
    def latitude(self, value):
        self._latitude = value

    @property
    def longitude(self):
        return self._longitude

    @longitude.setter
    def longitude(self, value):
        self._longitude = value

    @property
    def unit_in_address(self):
        return self._unit_in_address

    @unit_in_address.setter
    def unit_in_address(self, value):
        verify_instance_of(bool, value, "Unit in address must be a boolean")
        self._unit_in_address = value

    def __repr__(self):
        return self.get_single_line()

    def verify(self):
        verify_none_or_instance_of(str, self.unit, "Unit must be a str")
        verify_none_or_instance_of(str, self.street, "Street must be str")
        verify_none_or_instance_of(str, self.city, "City must be str")
        verify_none_or_instance_of(str, self.state, "State must be str")
        verify_none_or_instance_of(str, self.zip, "Zip must be str")
        verify_none_or_instance_of(str, self.country, "Country must be str")
        verify_none_or_instance_of(float, self.latitude, "Latitude must be float")
        verify_none_or_instance_of(float, self.longitude, "Longitude must be float")
        verify_none_or_within_range(self.latitude, -90, 90, "Latitude must be between -90 and 90")
        verify_none_or_within_range(self.longitude, -180, 180, "Longitude must be between -180 and 180")
        verify_all_none_or_not([self.latitude, self.longitude], "Latitude and Longitude must be both None or float"),


    def get_breakdown(self) -> dict:
        obj = dict()
        if self.unit is not None:
            obj['unit'] = self.unit
        if self.street is not None:
            obj['street'] = self.street
        if self.city is not None:
            obj['city'] = self.city
        if self.state is not None:
            obj['state'] = self.state
        if self.zip is not None:
            obj['zip'] = self.zip
        if self.country is not None:
            obj['country'] = self.country
        return obj

    def get_single_line(self) -> str:
        l = []
        if self.street is not None:
            l.append(self.street)
        if self.unit is not None and self.unit_in_address is True:
            l.append(self.unit)
        if self.city is not None:
            l.append(self.city)
        if self.state is not None:
            l.append(self.state)
        if self.zip is not None:
            l.append(self.zip)
        if self.country is not None:
            l.append(self.country)
        return ', '.join(l)



import json

import requests


class ShipdayClient:
    def __init__(self, *args, api_key, **kwargs, ):
        self._api_key = api_key
        self._timeout = kwargs['timeout'] if 'timeout' in kwargs else 1000
        self._base_url = 'https://api.shipday.com/'

    def __get_headers_(self):
        return {
            'Authorization': 'Basic {}'.format(self._api_key),
            'Content-Type': 'application/json',
        }

    def __get_api_key_(self):
        return self._api_key

    def __create_url_(self, suffix: str) -> str:
        return self._base_url + suffix

    def set_api_key(self, api_key: str):
        self._api_key = api_key

    def get(self, suffix: str):
        url = self.__create_url_(suffix)
        response = requests.get(url, headers=self.__get_headers_())
        return response.json()

    def post(self, suffix: str, data: dict):
        url = self.__create_url_(suffix)
        response = requests.post(url, json.dumps(data), headers=self.__get_headers_())
        return response.json()

    def put(self, suffix: str, data: dict):
        url = self.__create_url_(suffix)
        response = requests.put(url, json.dumps(data), headers=self.__get_headers_())
        return response.json()

    def delete(self, suffix: str):
        url = self.__create_url_(suffix)
        response = requests.delete(url, headers=self.__get_headers_())
        return response



from shipday.bo import PodType
from shipday.utils.verifiers import verify_instance_of, verify_none_or_instance_of
from shipday.exeptions.shipday_exeption import ShipdayException
from shipday.httpclient.shipdayclient import ShipdayClient
from shipday.order.address import Address
from datetime import datetime


class OnDemandDeliveryService:
    PATH = 'on-demand/'
    SERVICES_PATH = PATH + 'services'
    ESTIMATE_PATH = PATH + 'estimate/{order_id}'
    AVAILABILITY_PATH = 'third-party/availability'
    ASSIGN_PATH = PATH + 'assign'
    DETAILS_PATH = PATH + 'details/{order_id}'
    CANCEL_PATH = PATH + 'cancel/{order_id}'

    def __init__(self, *args, api_key, **kwargs):
        self.httpclient = ShipdayClient(*args, api_key=api_key, **kwargs)

    def get_services(self) -> list:
        res = self.httpclient.get(self.SERVICES_PATH)
        return res

    def get_active_services(self) -> list:
        services = self.get_services()
        active_services = [tp['name'] for tp in services if tp[tp['name']] is True]
        return active_services

    def estimate(self, order_id: int) -> dict:
        verify_instance_of(int, order_id, 'Order id must be integer')
        res = self.httpclient.get(self.ESTIMATE_PATH.format_map({'order_id': order_id}))
        return res

    def check_availability(self, *args, pickup_address: Address, delivery_address: Address,
                           delivery_time: datetime = None) -> dict:
        verify_instance_of(Address, pickup_address, 'Pickup address must be of type {}'.format(Address))
        verify_instance_of(Address, delivery_address, 'Delivery address must be of type {}'.format(Address))
        verify_none_or_instance_of(datetime, delivery_time, 'Delivery Time must be of type {}'.format(datetime))

        data = {
            'pickupAddress': pickup_address.get_single_line(),
            'deliveryAddress': delivery_address.get_single_line(),
            'pickup': pickup_address.get_breakdown(),
            'delivery': delivery_address.get_breakdown()
        }

        if pickup_address.latitude is not None and pickup_address.longitude is not None:
            data['pickupLatitude'] = pickup_address.latitude
            data['pickupLongitude'] = pickup_address.longitude

        if delivery_address.latitude is not None and delivery_address.longitude is not None:
            data['deliveryLatitude'] = delivery_address.latitude
            data['deliveryLongitude'] = delivery_address.longitude

        if delivery_time is not None:
            data['deliveryTime'] = delivery_time.isoformat()

        res = self.httpclient.post(self.AVAILABILITY_PATH, data)
        return res

    def assign(self, *args, order_id:int, service_name: str, tip: float = 0, estimate_reference=None, contactless_delivery: bool = False, pod_types: list[PodType] = None, **kwargs) -> dict:
        verify_instance_of(int, order_id, 'Order id must be integer')
        verify_instance_of([int, float], tip, 'Tip must be a number')
        verify_none_or_instance_of(str, estimate_reference, 'Invalid Reference')

        if service_name not in self.get_active_services():
            raise ShipdayException('Service not available')
        data = {
            'name': service_name,
            'orderId': order_id,
            'tip': tip,
            'contactlessDelivery': contactless_delivery,
            'podTypes': [pod.value for pod in pod_types] if pod_types is not None else None
        }
        if estimate_reference is not None:
            data['estimateReference'] = estimate_reference

        res = self.httpclient.post(self.ASSIGN_PATH, data)
        return res

    def cancel(self, order_id: int) -> dict:
        verify_instance_of(int, order_id, 'Order id must be integer')
        res = self.httpclient.post(self.CANCEL_PATH.format_map({'order_id': order_id}), {})
        return res

    def get_details(self, order_id: int) -> dict:
        verify_instance_of(int, order_id, 'Order id must be integer')
        res = self.httpclient.get(self.DETAILS_PATH.format_map({'order_id': order_id}))
        return res



# Always prefer setuptools over distutils
# To use a consistent encoding
from codecs import open
from os import path, chdir

from setuptools import setup, find_packages

# The directory containing this file
here = path.abspath(path.dirname(__file__))

chdir(here)

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

# Version
version_contents = {}
with open(path.join(here, 'shipday', 'version.py'), encoding="utf-8") as f:
    exec(f.read(), version_contents)

# This call to setup() does all the work
setup(
    name='shipday',
    version=version_contents['VERSION'],
    description="Python library for Shipday API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://shipday.com/",
    author="Shipday",
    author_email="shahriar@shipday.com",
    license="MIT",
    classifiers=[
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules"
    ],
    keywords=["Shipday",
              "DoorDash",
              "Uber",
              "Delivery API",
              "Dispatch API",
              "DoorDash API",
              "Uber API",
              "Dispatch App",
              "Courier App",
              "Delivery Dispatch",
              "Delivery integration",
              "Delivery Management",
              "Dispatch Management",
              "Delivery Service Integration",
              "Local Delivery API"],
    packages=find_packages(exclude=["tests", "tests.*"]),
    include_package_data=True,
    install_requires=[
        'requests >= 2.20; python_version >= "3.0"'
    ],
    python_requires=">=3.6",
    setup_requires=["wheel"],
)



from shipday.carrier import CarrierRequest
from shipday.exeptions import ShipdayException
from shipday.httpclient.shipdayclient import ShipdayClient


class CarrierService:
    path = 'carriers/'

    def __init__(self, *args, api_key, **kwargs):
        self.httpclient = ShipdayClient(*args, api_key=api_key, **kwargs)

    def get_carriers(self):
        return self.httpclient.get(self.path)

    def add_carrier(self, request: CarrierRequest):
        request.verify()
        response = self.httpclient.post(self.path, request.get_body())
        if 'errorCode' in response:
            raise ShipdayException(response['errorMessage'])
        return response

    def delete_carrier(self, carrier_id: int):
        if type(carrier_id) is not int:
            raise ShipdayException('Provide a valid carrier id')
        response = self.httpclient.delete(self.path + str(carrier_id))

        return response



import json
from datetime import datetime

from shipday.exeptions import ShipdayException
from shipday.order.order_status import OrderStatus
from shipday.utils.verifiers import verify_none_or_instance_of
from shipday.order.customer import Customer
from shipday.order.pickup import Pickup
from shipday.order.order_item import OrderItem
from shipday.order.order_cost import OrderCost
from shipday.order.address import Address


class OrderQuery:

    customer_address = Address(street='Jefferson St', city='California', state='CA', country='USA')
    customer = Customer(name='customer', address=customer_address, email='customer@shipday.com',
                        phone_number='+1343523423')

    pickup_address = Address(street='Hacker way', city='California', state='CA', country='USA')
    pickup = Pickup(name='customer', address=pickup_address, phone_number='+134343534')

    item_1 = OrderItem(name='Pizza', unitPrice=2, quantity=7, add_ons='Extra cheese', detail='Signature Item')
    cost = OrderCost(tips=1.0, deliveryFee=5, total=20)
    def __init__(self, *args,
                 start_time: datetime = None, end_time: datetime = None, order_status: str = None,
                 start_cursor=None, end_cursor=None,
                 **kwargs):
        self._start_time = start_time
        self._end_time = end_time
        self._order_status = order_status
        self._start_cursor = start_cursor
        self._end_cursor = end_cursor

    @property
    def start_time(self):
        return self._start_time

    @start_time.setter
    def start_time(self, value):
        verify_none_or_instance_of(datetime, value, "Start time is not of type " + str(datetime))
        self._start_time = value

    @property
    def end_time(self):
        return self._end_time

    @end_time.setter
    def end_time(self, value):
        verify_none_or_instance_of(datetime, value, "End time is not of type " + str(datetime))
        self._end_time = value

    @property
    def order_status(self):
        return self._order_status

    @order_status.setter
    def order_status(self, value):
        verify_none_or_instance_of(str, value, "Invalid order status")
        if value not in OrderStatus._list_:
            raise ShipdayException("Invalid order status")
        self._order_status = value

    @property
    def start_cursor(self):
        return self._start_cursor

    @start_cursor.setter
    def start_cursor(self, value):
        verify_none_or_instance_of(int, value, "Start cursor is not integer")
        self._start_cursor = value

    @property
    def end_cursor(self):
        return self._end_cursor

    @end_cursor.setter
    def end_cursor(self, value):
        verify_none_or_instance_of(int, value, "End cursor is not integer")
        self._end_cursor = value

    def __repr__(self):
        return json.dumps(self.get_body())

    def verify(self):
        verify_none_or_instance_of(datetime, self._start_time, "Start time is not of type " + str(datetime))
        verify_none_or_instance_of(datetime, self._end_time, "End time is not of type " + str(datetime))
        verify_none_or_instance_of(str, self._order_status, "Invalid order status")
        if self._order_status not in OrderStatus.list_:
            raise ShipdayException("Invalid order status")
        verify_none_or_instance_of(int, self._start_cursor, "Start cursor is not integer")
        verify_none_or_instance_of(int, self._end_cursor, "End cursor is not integer")

    def get_body(self):
        obj = dict()
        if self.start_time is not None:
            obj['startTime'] = self.start_time.isoformat()
        if self.end_time is not None:
            obj['endTime'] = self.end_time.isoformat()
        if self.order_status is not None:
            obj['orderStatus'] = self.order_status
        if self.start_cursor is not None:
            obj['startCursor'] = self.start_cursor
        if self.end_cursor is not None:
            obj['endCursor'] = self.end_cursor

        return obj



class OrderStatus:
    ALREADY_DELIVERED = 'ALREADY_DELIVERED'
    NOT_ASSIGNED = 'NOT_ASSIGNED'
    NOT_ACCEPTED = 'NOT_ACCEPTED'
    NOT_STARTED_YET = 'NOT_STARTED_YET'
    STARTED = 'STARTED'
    PICKED_UP = 'PICKED_UP'
    READY_TO_DELIVER = 'READY_TO_DELIVER'
    FAILED_DELIVERY = 'FAILED_DELIVERY'
    ACTIVE = 'ACTIVE'
    INCOMPLETE = 'INCOMPLETE'

    _list_ = ['ALREADY_DELIVERED', 'NOT_ASSIGNED', 'NOT_ACCEPTED', 'NOT_STARTED_YET', 'STARTED',
              'PICKED_UP', 'READY_TO_DELIVER', 'FAILED_DELIVERY', 'ACTIVE', 'INCOMPLETE']

    @property
    def list_(self):
        return self._list_



from shipday.exeptions import ShipdayException
from shipday.httpclient.shipdayclient import ShipdayClient
from shipday.order import Order, OrderQuery
from shipday.utils.verifiers import verify_instance_of


class OrderService:
    PATH = 'orders/'
    EDIT_PATH = 'order/edit/{order_id}'
    ASSIGN_PATH = PATH + 'assign/{order_id}/{carrier_id}'
    QUERY_PATH = PATH + 'query/'

    def __init__(self, *args, api_key, **kwargs):
        self.httpclient = ShipdayClient(*args, api_key=api_key, **kwargs)

    def get_orders(self) -> list:
        return self.httpclient.get(self.PATH)

    def get_order(self, order_number: str) -> list:
        if order_number is None:
            raise ShipdayException('Order number can not be None')
        response = self.httpclient.get(self.PATH + str(order_number))
        return response

    def insert_order(self, request: Order):
        request.verify()
        response = self.httpclient.post(self.PATH, request.get_body())
        if 'errorCode' in response:
            raise ShipdayException(response['errorMessage'])
        return response

    def edit_order(self, order_id: int, request: Order):
        request.verify()
        payload = request.get_body()
        payload['orderId'] = order_id
        response = self.httpclient.put(
            self.EDIT_PATH.format_map(
                {'order_id': order_id}
            ),
            payload
        )
        if 'errorCode' in response:
            raise ShipdayException(response['errorMessage'])
        return response

    def delete_order(self, order_id: int):
        verify_instance_of(int, order_id, 'Order id must be integer')

        response = self.httpclient.delete(self.PATH + str(order_id))
        return response

    def assign_order(self, order_id: int, carrier_id: int):
        verify_instance_of(int, order_id, 'Order id must be integer')
        verify_instance_of(int, carrier_id, 'Carrier id must be integer')

        response = self.httpclient.put(self.ASSIGN_PATH.format_map({'order_id': order_id, 'carrier_id': carrier_id}),
                                       {})
        return response

    def query(self, query: OrderQuery):
        if type(query) is not OrderQuery:
            raise ShipdayException("Query is not of type " + str(OrderQuery))
        response = self.httpclient.post(self.QUERY_PATH, query.get_body())
        return response
